/*------------------------------------------------------------------------------
    File        : customsp.p
    Description : Super procedure for site-specific additions
    Author(s)   : Thomas Mercer-Hursh
    Created     : Thu Jan 17 10:22:07 PST 2008
    Notes       : Use this procedure to add site-specific code.
    
NOTE: NO DEFAULT VERSION
THIS IS KAL TIRE VERSION, PROVIDED AS AN EXAMPLE

For simplicity, retain all procedure definitions as provided so that these
can be run by standard code at appropriate places without modification.
If the particular element does not apply, just delete the contents of the 
procedure so that it becomes a no-op, but leave the shell intact.
  ----------------------------------------------------------------------------*/
using com.cintegrity.Lib.Exception.*.
/* ***************************  Definitions  ******************************** */
                                                                   /* Classes */
define variable lobExceptionStack as class SimpleExceptionStack no-undo.
                                                               /* "Parameters */
define variable lchMenuProgSourceFile as character no-undo initial "MenuDescriptions.txt".  
                              /* Select one option according to the target OS */
define variable lchNewLine as character no-undo.
/*
lchNewLine = chr(10).                                             /* For Unix */
*/
lchNewLine = chr(13) + chr(10).                                /* For Windows */
                                                       
                                                               /* Package IDs */
define variable linUIModelPackageID as integer no-undo.
                                                               /* Temp-tables */
define temp-table ttPackage no-undo                   /* Package Descriptions */
    field chPackageName as character
    field chNotes as character
    index ixByName is unique primary chPackageName
    index ixByDescription chNotes
    .
define temp-table ttProgram no-undo                          /* Program Notes */
    field chProgramName as character
    field chNotes as character
    field chAlias as character
    index ixByName is unique primary chProgramName
    .
define temp-table ttMenu no-undo                                     /* Menus */
    field chMenuID as character
    field chMenuName as character
    field inMenuObjectID as integer
    index ixByID is unique primary chMenuID
    .
define temp-table ttMenuEntry no-undo                           /* Menu Items */
    field chMenuID as character
    field chOrder as character
    field chEntryText as character
    field chActionType as character
    field chActionTarget as character
    field inMenuEntryObjectID as integer
    index ixByMenuIDOrder is unique primary chMenuID chOrder
    .    
define temp-table ttMenuPackage no-undo
    field chType as character
    field chPackageName as character
    field inPackageID as integer
    field inObjectID as integer
    index ixByName as primary unique chType chPackageName
    .    
/* *****************************  Functions ********************************* */
function sfCompileName returns character ( 
    input ipchProgramName as character ) 
    in super. 
/* ***************************  Main Block  ********************************* */
              /* Do anything here to read custom data, else leave empty */
procedure spInitializeCustom:
      
  define buffer lbfPackage for DictSource.flx_module.
  define buffer lbfProgram for DictSource.flx_program.
  define buffer lbfMenu for DictSource.gen_menu.
  define buffer lbfMenuEntry for DictSource.gen_mitem.
     
  define variable iinEntry as integer no-undo.
  define variable ichProgramName as character  no-undo.
  define variable ichNotes as character  no-undo.
  define variable iinError as integer no-undo.
  define variable ichDiskFile as character no-undo.
                  /* Instantiate exception handler - leave in if used anywhere*/
  lobExceptionStack = new SimpleExceptionStack().
  lobExceptionStack:trace( "Start of Custom Preprocessing at " + string(now) ).
                                  /* Find base package for building UML menus */
  run spFindUIModelPackage (
      output linUIModelPackageID
      ).
  if lobExceptionStack:numExceptions( 1 ) gt 0
  then do:
    lobExceptionStack:pushException( 801, 1, "Fatal Error in com.cintegrity.ADBtoUML.code.codemain.p - aborting" ).
    run CleanUp.
    return.
  end. 
                                       /* Build table of Package Descriptions */
  for each lbfPackage no-lock:
    create ttPackage.
    assign
        ttPackage.chPackageName = lbfPackage.directory
        ttPackage.chNotes = lbfPackage.module
        no-error.
    if error-status:error
    then do:
      lobExceptionStack:pushException( 802, 2, "Load of Package descriptions failed for " + lbfPackage.directory + "/" + lbfPackage.module ).
      do iinError = 1 to error-status:num-messages:
        lobExceptionStack:pushException( 803, 4, error-status:get-message( iinError ) ).
      end.     
    end.
  end. 
                                              /* Build table of Program Notes */
  for each lbfProgram no-lock:
    if lbfProgram.ProgType = "ossource" then next.           /* shell scripts */
  
    find ttPackage no-lock
        where ttPackage.chNotes eq lbfProgram.Module
        no-error.
    if not available( ttPackage )
    then do:
      create ttPackage.
      assign 
          ttPackage.chPackageName = substring(lbfProgram.FileName,1,3)
          ttPackage.chNotes = lbfProgram.Module
          .
    end.
    create ttProgram.
    assign 
        ttProgram.chProgramName = lbfProgram.FileName
        ttProgram.chAlias = lbfProgram.FileName + " - " + lbfProgram.Description
        .
    if lbfProgram.Description ne ?
    then ttProgram.chNotes = "Description: " + lbfProgram.Description + lchNewLine.
    if lbfProgram.ProgGroup ne ? 
    then ttProgram.chNotes = ttProgram.chNotes + "ProgGroup: " + lbfProgram.ProgGroup + lchNewLine.
    if lbfProgram.ProgType ne ? 
    then ttProgram.chNotes = ttProgram.chNotes + "ProgType: " + lbfProgram.ProgType + lchNewLine.
    if lbfProgram.Prog-Notes ne ? 
    then ttProgram.chNotes = ttProgram.chNotes + "Notes: " + lbfProgram.Prog-Notes + lchNewLine.
    if lbfProgram.Text-Desc[1] ne ? 
    then ttProgram.chNotes = ttProgram.chNotes + "TextDesc: " + lbfProgram.Text-Desc[1].
    do iinEntry = 2 to 16:
      if      lbfProgram.Text-Desc[iinEntry] ne ""
          and lbfProgram.Text-Desc[iinEntry] ne ?
      then ttProgram.chNotes = ttProgram.chNotes + " " + lbfProgram.Text-Desc[iinEntry].
    end.
  end.
                             /* Add descriptions of constructed menu programs */
  ichDiskFile = search( lchMenuProgSourceFile ).
  if ichDiskFile eq ?
  then do:
    lobExceptionStack:pushException( 804, 1, "Cannot find " + lchMenuProgSourceFile + " - aborting" ).
    run CleanUp.
    return.
  end.
  input from value( ichDiskFile ) no-echo.
  repeat:
    import ichProgramName ichNotes.
    create ttProgram.
    assign
        ttProgram.chProgramName = ichProgramName
        ttProgram.chNotes = ichProgramName + " - " + ichNotes 
        .
  end.
  input close.

                                      /* Build Tables of Menus and Menu Items */
  for each lbfMenu no-lock:
    if     lbfMenu.MenuID eq ""
        or lbfMenu.MenuName eq "a"
        or lbfMenu.Entity eq ""
    then next.
    create ttMenu.
    assign
        ttMenu.chMenuID = lbfMenu.MenuID
        ttMenu.chMenuName = lbfMenu.MenuName
        .
  end.
  
  for each lbfMenuEntry no-lock
      where lbfMenuEntry.MenuType ne ""
      :
    create ttMenuEntry.
    assign
        ttMenuEntry.chMenuID = lbfMenuEntry.MenuID
        ttMenuEntry.chOrder = string(lbfMenuEntry.MenuOrder,"9999")
        ttMenuEntry.chEntryText = lbfMenuEntry.ItemDesc
        ttMenuEntry.chActionType = lbfMenuEntry.MenuType
        .
    case ttMenuEntry.chActionType:
        when "Submenu" then ttMenuEntry.chActionTarget = lbfMenuEntry.FileName.
        when "Program" then ttMenuEntry.chActionTarget = lbfMenuEntry.FilePath.
        when "Quit"    then ttMenuEntry.chActionTarget = "QUIT".
    end case.
  end.

end procedure.
/* *********************** Procedures *************************************** */
                                                     /* Get Package for Table */
procedure spGetTablePackage:
  define input parameter ipchTableName as character no-undo.
  define output parameter opchPackageName as character no-undo.

  if ipchTableName begins "_"
  then opchPackageName = "Metaschema".
  else do:
                                           /* Return "Application" by default */
    opchPackageName = entry(1, ipchTableName, "_" ).
  end.    
  
end procedure.
/* -------------------------------------------------------------------------- */
                                                   /* Get custom Package Note */
                                                      /* Return "" by default */                                                 
procedure spGetPackageNote:
  define input  parameter ipchPackageName as character  no-undo.
  define output parameter opchPackageNote as character  no-undo.
                                      /* Return custom note or blank */
  find ttPackage no-lock
      where ttPackage.chPackageName eq ipchPackageName
      no-error.
  if available( ttPackage )
  then opchPackageNote = ttPackage.chNotes.
  else opchPackageNote = "".
end procedure.
/* -------------------------------------------------------------------------- */
                                                   /* Get custom Program Note */
                                                      /* Return "" by default */
procedure spGetProgramNotes:
  define input  parameter ipchProgramName as character  no-undo.
  define output parameter opchProgramNote as character  no-undo.
  
  define variable iinSlashDot as integer no-undo.
  define variable ichBaseName as character no-undo.
  
  if ipchProgramName begins "analyst" then return.
  iinSlashDot = r-index( ipchProgramName, "/").
  if iinSlashDot gt 0
  then ichBaseName = substring( ipchProgramName, iinSlashDot + 1 ).
  else ichBaseName = ipchProgramName.
  if num-entries( ichBaseName, "." ) gt 0
  then ichBaseName = entry(1, ichBaseName, "." ).
    
  find ttProgram no-lock
      where ttProgram.chProgramName eq ichBaseName
      no-error.
  if available( ttProgram )
  then opchProgramNote = ttProgram.chNotes.
  else do:
    opchProgramNote = "".
    lobExceptionStack:pushException( 811, 4, "No program notes found for " + ipchProgramName + " using basename " + ichBaseName ).
  end.  
end procedure.
/* -------------------------------------------------------------------------- */
                                                  /* Get custom Program Alias */
                                                      /* Return "" by default */
procedure spGetProgramAlias:
  define input  parameter ipchProgramName as character  no-undo.
  define output parameter opchProgramAlias as character  no-undo.
  
  define variable iinSlashDot as integer no-undo.
  define variable ichBaseName as character no-undo.
  
  run BaseName (
      input ipchProgramName,
      output ichBaseName
      ).
      
  if not ipchProgramName begins "analyst"
  then do:
    find ttProgram no-lock
        where ttProgram.chProgramName eq ichBaseName
        no-error.
    if available( ttProgram )
    then opchProgramAlias = ttProgram.chAlias.
  end.
  else do:
    if ipchProgramName begins "analyst/m_"
    then do:
      ichBaseName = replace( ichBaseName, "m_", "" ).
      find ttMenu
          where ttMenu.chMenuID eq ichBaseName
          no-error.
      if available ( ttMenu )
      then opchProgramAlias = ichBaseName + " - " + ttMenu.chMenuName.
    end.
    else if ipchProgramName begins "analyst/lookup"
    then do:
      opchProgramAlias = ichBaseName.
    end.
  end.    
end procedure.
/* -------------------------------------------------------------------------- */
                                                   /* Get custom Program Path */
                                                      /* Return "" by default */                                                 
procedure spGetProgramPath:
  define input  parameter ipchProgramName as character  no-undo.
  define output parameter opchProgramPath as character  no-undo.
                                       /* Return full path name note or blank */
  opchProgramPath = search( ipchProgramName ).
  if opchProgramPath eq ?
  then opchProgramPath = "".

end procedure.
/* -------------------------------------------------------------------------- */
                               /* Additions to model after code base is built */
procedure spCustomAdditions:
  
  define variable iinMenuStructurePackageID as integer no-undo.
  define variable iinMenusPackageID as integer no-undo.
  define variable iinFunctionalUnitsPackageID as integer no-undo.
  define variable iinFunctionalUnitGroupPackageID as integer no-undo.
  define variable iinMenuGroupPackageID as integer no-undo.

  define variable iinQuitObjectID as integer no-undo.
  define variable iinConnectorID as integer no-undo.
  define variable iinObjectID as integer no-undo.
  define variable iinMenuGroupObjectID as integer no-undo.
  define variable iinMenuPackageID as integer no-undo.

  define variable ichNotes as character no-undo.
  define variable ichProgramPathName as character no-undo.
  define variable ichMenuPackage as character no-undo.
  define variable ichProgramUnitName as character no-undo.
  define variable ichCompileUnitName as character no-undo.
  define variable iinSlash as integer no-undo.
  define variable ichBaseName as character no-undo.

  define buffer ibfMenu for ttMenu.

  lobExceptionStack:trace( "Start of Custom Postprocessing at " + string(now) ).
  run spCreatePackage (
      input "Menu Structures",
      input linUIModelPackageID,
      output iinMenuStructurePackageID
      ).
  run spCreateObject (
      input "Package",
      input "Menu Structures",
      input iinMenuStructurePackageID,
      input 0,
      input "",
      input "",
      input "",
      output iinObjectID
      ).
                                                 /* Create Packages for Menus */
  run spCreatePackage (
      input "Menus",
      input iinMenuStructurePackageID,
      output iinMenusPackageID
      ).
  run spCreateObject (
      input "Package",
      input "Menus",
      input iinMenusPackageID,
      input 0,
      input "",
      input "",
      input "",
      output iinObjectID
      ).
  if iinObjectID eq ?
  then do:
    lobExceptionStack:pushException( 821, 2, "Something really wrong in Custom Additions" ).
    return.          
  end.      
                                                                          
  for each ttMenu:
                                        /* Create/find package for menu group */
    ichMenuPackage = substring( ttMenu.chMenuID, 1, 1 ) + "/" + substring( ttMenu.chMenuID, 2, 2 ).

    run FindPackage(
        input "Menus",
        input ichMenuPackage,
        input iinMenusPackageID,
        input "",
        output iinMenuGroupPackageID,
        output iinMenuGroupObjectID
        ).
    if     iinMenuGroupObjectID eq ?
        or iinMenuGroupObjectID eq 0
    then do:
      lobExceptionStack:pushException( 822, 2, "Failure to FindPackage for " + ichMenuPackage ).
    end.
                                        /* Create package for individual menu */
    run spCreatePackage (
        input ttMenu.chMenuID + " " + ttMenu.chMenuName,
        input iinMenuGroupPackageID,
        output iinMenuPackageID
        ).
    if     iinMenuPackageID eq ?
        or iinMenuPackageID eq 0
    then do:
      lobExceptionStack:pushException( 823, 2, "Failure to Create Package for " + ttMenu.chMenuID ).
    end.
    run spCreateObject (
        input "Package",
        input ttMenu.chMenuID + " " + ttMenu.chMenuName,
        input iinMenuPackageID,
        input 0,
        input "oeMenu",
        input "",
        input ttMenu.chMenuID + " " + ttMenu.chMenuName,
        output ttMenu.inMenuObjectID
        ).  
    if     ttMenu.inMenuObjectID eq ?
        or ttMenu.inMenuObjectID eq 0
    then do:
      lobExceptionStack:pushException( 824, 2, "Failure to Create Object for " + ttMenu.chMenuID ).
    end.
                                    /* Create list of menu contents for notes */
    ichNotes = "".
    for each ttMenuEntry
        where ttMenuEntry.chMenuID = ttMenu.chMenuID
        :
      ichNotes = ichNotes + substitute( "&1. &2 ( &3=&4 )",ttMenuEntry.chOrder,ttMenuEntry.chEntryText,ttMenuEntry.chActionType,ttMenuEntry.chActionTarget) + lchNewLine.
    end.
    run spAddPackageNotes (
        input iinMenuPackageID,
        input ichNotes 
        ). 
    run spAddObjectNotes (
        input ttMenu.inMenuObjectID,
        input ichNotes 
        ). 
                                          /* Link to analyst/m_nnnnn programs */
    run spFindObject (
        input "Package",
        input "oeCompileUnit",
        input "analyst/m_" + ttMenu.chMenuID + ".r",
        output iinObjectID
        ).
    if     iinObjectID eq ?
        or iinObjectID eq 0
    then do:
      lobExceptionStack:pushException( 825, 2, "Failure to FindObject for analyst/m_" + ttMenu.chMenuID + ".r" ).
    end.
    else do:
      run spCreateConnector (
          input "Dependency",
          input "",
          input "",
          input "",
          input "",
          input ttMenu.inMenuObjectID,
          input iinObjectID,
          input "oeMenuEquivalent",
          output iinConnectorID
          ).            
      if     iinConnectorID eq ?
          or iinConnectorID eq 0
      then do:
        lobExceptionStack:pushException( 826, 2, "Failure to Create Connector for analyst/m_" + ttMenu.chMenuID + ".r" ).
      end.
      run spAddObjectNotes (
          input iinObjectID,
          input ichNotes 
          ).
    end.
                                                      /* Add notes also to PU */
    run spFindObject (
        input "Component",
        input "oeProgram",
        input "analyst/m_" + ttMenu.chMenuID + ".p",
        output iinObjectID
        ).
    if     iinObjectID eq ?
        or iinObjectID eq 0
    then do:
      lobExceptionStack:pushException( 827, 2, "Failure to FindObject for analyst/m_" + ttMenu.chMenuID + ".p" ).
    end.
    else do:
      run spAddObjectNotes (
          input iinObjectID,
          input ichNotes 
          ).
    end.  
  end.
                                      /* Create Packages for Functional Units */
  run spCreatePackage (
      input "Functional Units",
      input iinMenuStructurePackageID, 
      output iinFunctionalUnitsPackageID
      ).
  run spCreateObject (
      input "Package",
      input "Functional Units",
      input iinFunctionalUnitsPackageID,
      input 0,
      input "",
      input "",
      input "",
      output iinObjectID
      ).
   if     iinObjectID eq ?
       or iinObjectID eq 0
   then do:
     lobExceptionStack:pushException( 828, 1, "Failure to Create Object for FunctionalUnits" ).
     return.
   end.
                                                    /* Create object for QUIT */
   run spCreateObject (
      input "Component",
      input "QUIT",
      input iinFunctionalUnitsPackageID,
      input 0,
      input "oeFunctionalUnit",
      input "Generic standin for menu exit",
      input "Generic standin for menu exit",
      output iinQuitObjectID
      ).      
   if     iinObjectID eq ?
       or iinObjectID eq 0
   then do:
     lobExceptionStack:pushException( 829, 1, "Failure to CreateObject for Quit" ).
     return.
   end. 
                       /* Create functional units for each Program menu entry */ 
   for each ttMenuEntry
       where ttMenuEntry.chActionType eq "Program"
       :
     if ttMenuEntry.chActionTarget eq "" then next. 
     iinSlash = r-index( ttMenuEntry.chActionTarget, "/" ).
     if iinSlash eq 0
     then do:
       lobExceptionStack:pushException( 830, 1, "No slash in action type for menu " + ttMenuEntry.chEntryText ).
       next.
     end.  
     ichProgramPathName = substring( ttMenuEntry.chActionTarget, 1, iinSlash - 1 ).
     
     run FindPackage(
         input "Functional Unit",
         input ichProgramPathName,
         input iinFunctionalUnitsPackageID,
         input "",
         output iinFunctionalUnitGroupPackageID,
         output iinObjectID
         ).
     if     iinObjectID eq ?
         or iinObjectID eq 0
     then do:
       lobExceptionStack:pushException( 831, 1, "Failure to FindPackage for FU package + " + ichProgramPathName ).
     end.
     else do:
       run BaseName (
          input ttMenuEntry.chActionTarget,
          output ichBaseName
          ).
       run spCreateObject (
          input "Component",
          input ttMenuEntry.chActionTarget,
          input iinFunctionalUnitGroupPackageID,
          input 0,
          input "oeFunctionalUnit",
          input ttMenuEntry.chEntryText,   /* should contain list of contents */
          input ichBaseName + " - " + ttMenuEntry.chEntryText,
          output ttMenuEntry.inMenuEntryObjectID
          ).  
       if     ttMenuEntry.inMenuEntryObjectID eq ?
           or ttMenuEntry.inMenuEntryObjectID eq 0
       then do:
         lobExceptionStack:pushException( 832, 2, "Failure to Create Object for FU + " + ichProgramPathName + "/" + ttMenuEntry.chActionTarget ).
         next.
       end.
                                                      /* Link to compile unit */
       ichCompileUnitName = sfCompileName( ttMenuEntry.chActionTarget ). 
       run spFindObject (
           input "Package",
           input "oeCompileUnit",
           input ichCompileUnitName,
           output iinObjectID
           ).
       if     iinObjectID eq ?
           or iinObjectID eq 0
       then do:
         lobExceptionStack:pushException( 833, 2, "Failure to Find Object for FU + " + ichCompileUnitName ).
         next.
       end.
       run spCreateConnector (
           input "Dependency",
           input "",
           input "",
           input "",
           input "",
           input ttMenuEntry.inMenuEntryObjectID,
           input iinObjectID,
           input "oeFunctionalLink",
           output iinConnectorID
           ).
       if     iinConnectorID eq ?
           or iinConnectorID eq 0
       then do:
         lobExceptionStack:pushException( 834, 2, "Failure to Create Connector for FU + " + ichProgramPathName + "/" + ttMenuEntry.chActionTarget + ".r" ).
         next.
       end.
    end.        
  end.    
                              /* Link menus and submenus and functional units */
  for each ttMenu:
    for each ttMenuEntry
        where ttMenuEntry.chMenuID eq ttMenu.chMenuID
        :
      case ttMenuEntry.chActionType:
        when "Submenu"
          then do:
            find ibfMenu
                where ibfMenu.chMenuID eq ttMenuEntry.chActionTarget
                no-error.
            if available ( ibfMenu )
            then do:    
              run spCreateConnector (
                  input "ControlFlow",
                  input "",
                  input "",
                  input "",
                  input "",
                  input ttMenu.inMenuObjectID,
                  input ibfMenu.inMenuObjectID,
                  input "oeMenuLink",
                  output iinConnectorID
                  ).
             if     iinConnectorID eq ?
                 or iinConnectorID eq 0
             then do:
               lobExceptionStack:pushException( 835, 2, "Failure to Create Connector from Menu " + ttMenu.chMenuID + " to Submenu + " + ibfMenu.chMenuID ).
               next.
             end.
            end.
            else do:
              lobExceptionStack:pushException( 836, 2, "Submenu target = " + ttMenuEntry.chActionTarget + " with no corresponding menu for " + ttMenuEntry.chMenuID ).
            end.
          end.
        when "Program"
          then do:
            if      ttMenuEntry.inMenuEntryObjectID ne ?
                and ttMenuEntry.inMenuEntryObjectID ne 0
            then do:
              run spCreateConnector (
                  input "ControlFlow",
                  input "",
                  input "",
                  input "",
                  input "",
                  input ttMenu.inMenuObjectID,
                  input ttMenuEntry.inMenuEntryObjectID,
                  input "oeMenuLink",
                  output iinConnectorID
                  ).
               if     iinConnectorID eq ?
                   or iinConnectorID eq 0
               then do:
                 lobExceptionStack:pushException( 837, 2, "Failure to Create Connector from Menu " + ttMenu.chMenuID + " to Program " + ttMenuEntry.chActionTarget ).
                 next.
               end.
             end.
          end.
        when "Quit"
          then do:
            run spCreateConnector (
                input "ControlFlow",
                input "",
                input "",
                input "",
                input "",
                input ttMenu.inMenuObjectID,
                input iinQuitObjectID,
                input "oeMenuLink",
                output iinConnectorID
                ).
             if     iinConnectorID eq ?
                 or iinConnectorID eq ?
             then do:
               lobExceptionStack:pushException( 838, 2, "Failure to Create Connector from Menu " + ttMenu.chMenuID + " to Quit" ).
               next.
             end.
          end.
        otherwise
          do:
            lobExceptionStack:pushException( 839, 2, "Unexpected gen_mitem action type = " + ttMenuEntry.chActionType + " for " + ttMenuEntry.chMenuID ).
          end.
      end case.
 
    end.
  end.
                                                    /* Special Postprocessing */
  run com/cintegrity/ABLtoUML/site/postprocess.p.    
end procedure.
/* ************************************************************************** */
                                                                 /* Clean up */
procedure spCustomDestruct:
  delete object lobExceptionStack.
  lobExceptionStack = ?.
  empty temp-table ttProgram.
  empty temp-table ttPackage.
  empty temp-table ttMenu.
  empty temp-table ttMenuEntry.
end procedure.
/* ************************  Helper Procedures ****************************** */
                                                /* Find/Create TT for Package */
procedure FindPackage:
    define input parameter ipchType as character  no-undo.
    define input parameter ipchPackageName as character no-undo.
    define input parameter ipinParentPackageID as integer no-undo.
    define input parameter ipchStereotype as character no-undo.
    define output parameter opinPackageID as integer no-undo.
    define output parameter opinObjectID as integer no-undo.
  
  define variable ichPackageParentName as character  no-undo.
  define variable ichPackageBaseName as character no-undo.
  define variable iinLocalParentPackageID as integer no-undo.
  define variable iinLocalParentObjectID as integer no-undo.
  define variable iinLocalPackageID as integer no-undo.
  define variable iinLocalObjectID as integer no-undo.
  define variable ichNotes as character  no-undo.
  
  find ttMenuPackage
      where ttMenuPackage.chType eq ipchType 
      and   ttMenuPackage.chPackage eq ipchPackageName 
      no-error.

  if not available( ttMenuPackage )
  then do:
    run ParsePackage (
        input ipchPackageName,
        output ichPackageParentName,
        output ichPackageBaseName
        ).
    if ichPackageParentName ne ""
    then do:
      run FindPackage (
          input ipchType,
          input ichPackageParentName,
          input ipinParentPackageID,
          input "",
          output iinLocalParentPackageID,
          output iinLocalParentObjectID
          ).
    end.
    else do:
      iinLocalParentPackageID = ipinParentPackageID.
    end.
                                                            /* Create package */
    run spCreatePackage (
        input ipchPackageName,
        input iinLocalParentPackageID,
        output iinLocalPackageID
        ).
     if iinLocalPackageID eq ?
     then do:
       lobExceptionStack:pushException( 891, 2, "Failure to CreatePackage for " + ipchPackageName ).
       next.
     end.
                                            /* Check for custom package Notes */
    run spGetPackageNote (
        input ipchPackageName,
        output ichNotes
        ).
    if ichNotes ne ""
    then do:
      run spAddPackageNotes (
          input iinLocalPackageID,
          input ichNotes
          ).
    end.
                                               /* Create corresponding object */
    run spCreateObject (
        input "Package",
        input ipchPackageName,
        input iinLocalPackageID,
        input 0,
        input ipchStereotype,
        input "",
        input "",
        output iinLocalObjectID
        ).    
     if iinLocalObjectID eq ?
     then do:
       lobExceptionStack:pushException( 892, 2, "Failure to CreateObject for " + ipchPackageName ).
       next.
     end.
                                                   /* Create Temp Table entry */
    create ttMenuPackage.
    assign
        ttMenuPackage.chType = ipchType
        ttMenuPackage.chPackageName = ipchPackageName
        ttMenuPackage.inPackageID = iinLocalPackageID
        ttMenuPackage.inObjectID = iinLocalObjectID
        .
  end.  
  assign
      opinPackageID = ttMenuPackage.inPackageID
      opinObjectID = ttMenuPackage.inObjectID
      .
end procedure.
/* -------------------------------------------------------------------------- */
procedure ParsePackage:
  define input  parameter ipchPackageFullName as character  no-undo.
  define output parameter opchPackageParent as character  no-undo.
  define output parameter opchPackageBaseName as character  no-undo.
  
  define variable iinSlash as integer    no-undo.
  
  iinSlash = r-index(ipchPackageFullName,"/").
  if iinSlash eq 0
  then iinSlash = r-index(ipchPackageFullName,"~\").
  if iinSlash eq 0
  then
    assign
        opchPackageParent = ""
        opchPackageBaseName = ipchPackageFullName
        .
  else
    assign
        opchPackageParent = substring(ipchPackageFullName, 1, iinSlash - 1 )
        opchPackageBaseName = substring(ipchPackageFullName, iinSlash + 1 )
        .
end procedure.    
/* -------------------------------------------------------------------------- */
procedure BaseName:
  define input parameter ipchFullName as character no-undo.
  define output parameter opchBaseName as character no-undo.
  
  define variable iinSlash as integer no-undo.
  
  iinSlash = r-index( ipchFullName, "/").
  if iinSlash gt 0
  then opchBaseName = substring( ipchFullName, iinSlash + 1 ).
  else opchBaseName = ipchFullName.
  if num-entries( opchBaseName, "." ) gt 0
  then opchBaseName = entry(1, opchBaseName, "." ).
end procedure.     
/************************** End of Program ************************************/